class ServiceEnum:
    ADMIN_TOOLS = 'admin_tools'
    ATTRIBUTION_MODELS = 'attribution_models'
    CLIENTS = 'clients'
    DATALAB = 'datalab_monitoring'
    FEED_DRIVEN_ADS = 'feed_driven_ads'
    LAB = 'lab'
    NOTIFICATIONS = 'notification'
    MCT_ALERTS = 'alerts_mct'
    PILOT = 'pilot_dashboard'
    SEARCH_TERMS = 'search_term'
    SMART_FEEDS_ING = 'smart_feeds_ingestions'
    SMART_FEEDS_PROC = 'smart_feeds_processings'
    SMART_FEEDS_EXP = 'smart_feeds_export'
    SMART_FEEDS_PF = 'smart_feeds_product_flow'
    SMART_FEEDS_AF = 'smart_feeds_audience_flow'
    SMART_FEEDS_PERFORMANCE_DASHBOARD = 'smart_feeds_performance_dashboard'
    TRACKING = 'adscale_user_activity_tracker'
    USERS = 'users'
    USERS_RIGHTS = 'users_rights'
    BUDGET_MONITORING = 'budget_monitoring'
